from flask import Blueprint, jsonify
import yaml

from config import *
import helpers.kubernetes as k8s
import requests

api_v1 = Blueprint("api_v1", __name__, url_prefix="/api/v1")


@api_v1.route("/")
@api_v1.route("/health")
def api_index():
    return "Stackspin API v1.0"

@api_v1.route("/environment")
def api_environment():
    environment = {
        "HYDRA_PUBLIC_URL": HYDRA_PUBLIC_URL,
        "KRATOS_PUBLIC_URL": KRATOS_PUBLIC_URL,
    }
    return jsonify(environment)

# We want to know if
# 1. A release has happened recently and is already deployed on this cluster.
# 2. A release has happened recently but has not yet been deployed on this
# cluster -- that will then probably happen automatically during the next
# maintenance window.
#
# To get the last release, we get the contents of the `VERSION` file from
# the main branch. The `VERSION` file is only updated as part of the release
# process.
#
# To find out how long ago the currently running version was deployed, we look
# at the `lastUpdateTime` of the stackspin `GitRepo` object on the cluster.
@api_v1.route("/info")
def api_info():
    # Get static info from configmap on cluster.
    static_info = k8s.get_kubernetes_config_map_data(
        "stackspin-static-info",
        "flux-system",
    )
    results = static_info

    # Get app versions from apps configmaps on cluster.
    results['appVersions'] = {}
    apps = k8s.get_kubernetes_config_map_data(
        "stackspin-apps",
        "flux-system",
    )
    for app, app_data in apps.items():
        data = yaml.safe_load(app_data)
        if 'version' in data:
            results['appVersions'][app] = data['version']
    apps_custom = k8s.get_kubernetes_config_map_data(
        "stackspin-apps-custom",
        "flux-system",
    )
    if apps_custom is not None:
        for app, app_data in apps_custom.items():
            data = yaml.safe_load(app_data)
            if 'version' in data:
                results['appVersions'][app] = data['version']

    # Get last update time of stackspin GitRepo object on the cluster; that
    # tells us when flux last updated the cluster based on changes in the
    # stackspin git repo.
    stackspin_repo = k8s.get_gitrepo('stackspin')
    results['lastUpdated'] = stackspin_repo['status']['artifact']['lastUpdateTime']

    # This is the branch (or other git ref, like tag or commit) that this
    # cluster follows.
    flux_ref = stackspin_repo['spec']['ref']
    # `flux_ref` is a structured object, though as far as we've seen always a
    # dict with a single entry. The key can be `branch` or `tag` or `commit`.
    # We reduce this to a single string git ref for simplicity in the
    # front-end.
    ref = next(iter(flux_ref.values()))
    results['followingGit'] = ref

    # Get latest released version from gitlab. Whether it's considered
    # "released" depends on which branch we're following, but usually that's
    # the `vX` "production" branch.
    git_release = requests.get(f"https://open.greenhost.net/stackspin/stackspin/-/raw/{ref}/VERSION").text.rstrip()
    results['lastRelease'] = git_release

    return jsonify(results)
